@@ -41,11 +41,12 @@ class BipartiteLongCollapsed(bpd.BipartiteLongBase):
         '''
         return bpd.BipartiteEventStudyCollapsed
 
-    def recollapse(self, copy=True):
+    def recollapse(self, drop_multiples=False, copy=True):
         '''
         Recollapse data by job spells (so each spell for a particular worker at a particular firm is one observation). This method is necessary in the case of biconnected data - it can occur that a worker works at firms A and B in the order A B A, but the biconnected components removes firm B. So the data is now A A, and needs to be recollapsed so this is marked as a stayer.
 
         Arguments:
+            drop_multiples (bool): if True, rather than collapsing over spells, drop any spells with multiple observations (this is for computational efficiency when computing biconnected components)
             copy (bool): if False, avoid copy
 
         Returns:
@@ -53,50 +54,63 @@ class BipartiteLongCollapsed(bpd.BipartiteLongBase):
         '''
         frame = pd.DataFrame(self, copy=copy)
 
-        # Sort data by i and t
-        frame = frame.sort_values(['i', 't1'])
+        # FIXME I assume if the data is in long-collapsed form it is already sorted
+        # # Sort data by i and t
+        # frame = frame.sort_values(['i', 't1'])
 
         # Add w
         if 'w' not in frame.columns:
             frame['w'] = 1
 
         # Introduce lagged i and j
-        frame['i_l1'] = frame['i'].shift(periods=1)
-        frame['j_l1'] = frame['j'].shift(periods=1)
+        i_l1 = frame['i'].shift(periods=1)
+        j_l1 = frame['j'].shift(periods=1)
 
         # Generate spell ids
-        new_spell = (frame['j'] != frame['j_l1']) | (frame['i'] != frame['i_l1']) # Allow for i != i_l1 to ensure that consecutive workers at the same firm get counted as different spells
-        frame['spell_id'] = new_spell.cumsum()
+        new_spell = (frame['j'] != j_l1) | (frame['i'] != i_l1) # Allow for i != i_l1 to ensure that consecutive workers at the same firm get counted as different spells
+        del i_l1, j_l1
+        spell_id = new_spell.cumsum()
+
+        # Quickly check whether a recollapse is necessary
+        if spell_id.iloc[-1] == len(frame):
+            if copy:
+                return self.copy()
+            return self
 
         # Aggregate at the spell level
-        spell = frame.groupby(['spell_id'])
-        # First, aggregate required columns
-        data_spell = spell.agg(
-            i=pd.NamedAgg(column='i', aggfunc='first'),
-            j=pd.NamedAgg(column='j', aggfunc='first'),
-            y=pd.NamedAgg(column='y', aggfunc='mean'),
-            t1=pd.NamedAgg(column='t1', aggfunc='min'),
-            t2=pd.NamedAgg(column='t2', aggfunc='max'),
-            w=pd.NamedAgg(column='w', aggfunc='sum')
-        )
-        # Next, aggregate optional columns
-        all_cols = self._included_cols()
-        for col in all_cols:
-            if col in self.columns_opt:
-                if self.col_dtype_dict[col] == 'int':
-                    for subcol in bpd.to_list(self.reference_dict[col]):
-                        data_spell[subcol] = spell[subcol].first()
-                if self.col_dtype_dict[col] == 'float':
-                    for subcol in bpd.to_list(self.reference_dict[col]):
-                        data_spell[subcol] = spell[subcol].mean()
+        spell = frame.groupby(spell_id)
 
-        collapsed_data = data_spell.reset_index(drop=True)
+        if drop_multiples:
+            data_spell = frame[spell['i'].transform('size') == 1]
+            data_spell.reset_index(drop=True, inplace=True)
+        else:
+            # First, aggregate required columns
+            data_spell = spell.agg(
+                i=pd.NamedAgg(column='i', aggfunc='first'),
+                j=pd.NamedAgg(column='j', aggfunc='first'),
+                y=pd.NamedAgg(column='y', aggfunc='mean'),
+                t1=pd.NamedAgg(column='t1', aggfunc='min'),
+                t2=pd.NamedAgg(column='t2', aggfunc='max'),
+                w=pd.NamedAgg(column='w', aggfunc='sum')
+            )
+            # Next, aggregate optional columns
+            all_cols = self._included_cols()
+            for col in all_cols:
+                if col in self.columns_opt:
+                    if self.col_dtype_dict[col] == 'int':
+                        for subcol in bpd.to_list(self.reference_dict[col]):
+                            data_spell[subcol] = spell[subcol].first()
+                    if self.col_dtype_dict[col] == 'float':
+                        for subcol in bpd.to_list(self.reference_dict[col]):
+                            data_spell[subcol] = spell[subcol].mean()
+
+            data_spell.reset_index(drop=True, inplace=True)
 
         # Sort columns
-        sorted_cols = sorted(collapsed_data.columns, key=bpd.col_order)
-        collapsed_data = collapsed_data[sorted_cols]
+        sorted_cols = sorted(data_spell.columns, key=bpd.col_order)
+        data_spell = data_spell.reindex(sorted_cols, axis=1, copy=False)
 
-        collapsed_frame = bpd.BipartiteLongCollapsed(collapsed_data)
+        collapsed_frame = bpd.BipartiteLongCollapsed(data_spell)
         collapsed_frame._set_attributes(self, no_dict=False)
 
         return collapsed_frame
@@ -132,7 +146,7 @@ class BipartiteLongCollapsed(bpd.BipartiteLongBase):
 
         # Sort columns
         sorted_cols = sorted(data_long.columns, key=bpd.col_order)
-        data_long = data_long[sorted_cols]
+        data_long = data_long.reindex(sorted_cols, axis=1, copy=False)
 
         self.logger.info('data uncollapsed to long format')
 
